/**
 * A program used to generate Horner equations and MSP430 code for
 * multiplication on micros that can only do shifts and adds.
 * 
 * This code copyright 2007 by Craig A. Lindley. Personal use is permitted.
 * Professional use must be licensed.
 *
 * @author Craig A. Lindley
 * @version 1.0
 * Last Update: 04/15/2007 
 */

/**
 * A class that holds a representation of a binary number along with
 * assorted metadata about the number. Two representations are needed
 * to hold a fixed point number. Ints are used for each bit so each can
 * take on the values of 1, 0 or -1. -1 values come into
 * play for CSD numbers.
 * 
 * @author Craig A. Lindley
 */
class Representation {
	
	private static final String CRLF 			= "\r\n";

	/**
	 * Class constructor creates storage for the representation of the
	 * specified number of bits and sets the the two bits of metadata
	 * to false.
	 * 
	 * @param numberBits number of bits in the representation
	 */
	public Representation(final int numberBits) {

		// Save incoming
		this.numberBits = numberBits;
		
		// Create a representation array
		representation = new int [numberBits];
		
		// Clear metadata 
		signBit = false;
		guardBit = false;
	}
	
	/**
	 * Determine if the representation has no bits set
	 * 
	 * @return true if all bits of representation are zero; false if not
	 */
	public boolean zero() {
		
		for (int i = 0; i < numberBits; i++) {
			if (representation[i] != 0) {
				return false;
			}
		}
		return true;
	}
	
	/**
	 * Performs a 1's compliment operation on the representation. This is done
	 * by inverting each bit.
	 */
	public void onesComp() {
		
		// Does representation have any one bits ?
		if (! zero()) {
			// Do 1's comp operation if so.
			for (int i = 0; i < numberBits; i++) {

				if (representation[i] == 0) {
					representation[i] = 1;
				} else {
					representation[i] = 0;
				}
			}
		}
	}
	
	/**
	 * Create a long value from the representation
	 * 
	 * @return long int representing the representation
	 */
	private long toLong() {

		long l = 0;

		for (int i = numberBits - 1; i >=0; i--) {
			l <<= 1;

			int b = representation[i];
			if (b != 0) {
				l += 1;
			}
		}
		return l;
	}
	
	/**
	 * Make the representation conform to the long value
	 * 
	 * @param value long value that the representation should conform to
	 */
	private void fromLong(final long value) {
		
		long l = value;
		
		for (int i = 0; i < numberBits; i++) {
			representation[i] = (int)(l & 1L);
			l >>= 1;
		}
	}
	
	/**
	 * Performs a 2's compliment operation on the representation. This is done
	 * by inverting each bit and adding one. This operation converts a positive
	 * representation into a negative one and vise versa.
	 */
	public void twosComp() {
		
		// Is there anything to do ?
		if (zero()) {
			return;
		}
		// Convert representation to a long
		long l = toLong();
		
		// Do 2's comp
		l = -l;
		
		// Convert long back to representation
		fromLong(l);
	}

	public int getNumberBits() {
		return numberBits;
	}
	
	public boolean getSignBit() {
		return signBit;
	}
	
	public void setSignBit(final boolean signBit) {
		this.signBit = signBit;
	}
	
	public boolean getGuardBit() {
		return guardBit;
	}
	
	public void setGuardBit(final boolean guardBit) {
		this.guardBit = guardBit;
	}
	
	public int [] getRepresentation() {
		return representation;
	}
	
	public void setRepresentation(final int [] representation) {
		this.representation = representation;
	}

	public int getBit(final int bitNumber) {
		
		return representation[bitNumber];
	}
	
	public void setBit(final int bitNumber, final int bit) {
		
		representation[bitNumber] = bit;
	}
	
	public String formatRepresentation(final boolean showMetadata) {
	
		StringBuffer sb = new StringBuffer();

		if (showMetadata) {
			sb.append("Signbit (true is negative): " + signBit + CRLF);
			sb.append("Guardbit: " + guardBit + CRLF);
		}
		// If representation is all zeros, substitute single 0
		if (zero()) {
			sb.append("0");

		} else {
		
			for (int i = numberBits - 1; i >= 0; i--) {
				int bit = representation[i];
				if (bit == 1) {
					sb.append("1");
				} else if (bit == 0){
					sb.append("0");
				} else {
					sb.append("-1");
				}
			}
		}
		return sb.toString();
	}
	
	public String toString() {
		
		return formatRepresentation(true);
	}
	
	// Instance data
	private int numberBits;
	private boolean signBit;
	private boolean guardBit;
	private int [] representation;
}

/**
 * Class used to convert a floating point number into a fixed point 
 * representation of a specified number of bits and to provide methods
 * for accessing the representation.
 * 
 * @author Craig A. Lindley
 */
class Holder {
		
	/**
	 *  Class constructor converts a double number into a binary representation 
	 *  with the specified number of bits.
	 *  
	 * @param numberBits number of bits to use in the binary representation
	 * @param number double number to convert into binary
	 */
	public Holder(final int numberBits, double number) {

		// Save incoming
		this.numberBits = numberBits;

		// Save info about sign of number. If the number is negative
		// record that fact and convert to positive number for conversion.
		if (number < 0.0) {
			negative = true;
			number = Math.abs(number);
		}
			
		// Create the representations for this number
		decimalRepresentation  = new Representation(numberBits);
		fractionRepresentation = new Representation(numberBits);
			
		// Work on the decimal and fractional part of the number separately
		int decimalPart = new Double(number).intValue();
		double fractionPart = number - decimalPart;
			
		// Do the decimal conversion
		int index = 0;
		int dnum = decimalPart;

		while (dnum > 0) {
			if (dnum % 2 != 0) {
				decimalRepresentation.setBit(index++, 1);
			} else {
				decimalRepresentation.setBit(index++, 0);
			}
			dnum /= 2;
		}
			
		// Do the fractional conversion
		index = numberBits - 1;
		double fnum = fractionPart;
		double binaryFraction = 0.5;

		while ((fnum > 0) && (index >= 0)) {
			if (fnum >= binaryFraction) {
				fractionRepresentation.setBit(index--, 1);
				fnum -= binaryFraction;
			} else {
				fractionRepresentation.setBit(index--, 0);
			}
			binaryFraction /= 2;
		}
		// If input number was negative, do 1's comp on decimal representation
		// 2's comp on fractional representation and set metadata indicating the
		// number is negative.
		if (negative) {
			if (fractionRepresentation.zero()) {
				decimalRepresentation.twosComp();
			} else {
				decimalRepresentation.onesComp();
			}
			fractionRepresentation.twosComp();
			decimalRepresentation.setSignBit(true);
			fractionRepresentation.setSignBit(true);
		}
	}
	
	/**
     * Convert internal representations into a String for display. Take into 
     * consideration the tristate nature of the representation. That is each
     * bit of the representation is in the set {0, 1, -1}. The -1 comes into
     * play when using CSD.
     * 
     * @return String containing the representation.
     */ 
	@Override
	public String toString() {
			
		StringBuffer sb = new StringBuffer();
		sb.append(decimalRepresentation.formatRepresentation(false));

		// Add the implied decimal point 
		sb.append(".");
			
		sb.append(fractionRepresentation.formatRepresentation(false));

		return sb.toString();
	}

	/**
	 * Get the number of bits in the representation
	 * 
	 * @return number of bits in the representation
	 */
	public int getNumberBits() {
		return numberBits;
	}

	/**
	 * Return a representation of the decimal portion of the floating point
	 * number.
	 * 
	 * @return a Representation object containing the representation
	 */
	public Representation getDecimalRepresentation() {
		return decimalRepresentation;
	}
	
	/**
	 * Set the decimal internal representation
	 *  
	 * @param representation object containing the representation
	 */
	public void setDecimalRepresentation(Representation representation) {
		decimalRepresentation = representation;
	}
	
	/**
	 * Return a representation of the fraction portion of the floating point
	 * number.
	 * 
	 * @return a Representation object containing the representation
	 */
	public Representation getFractionRepresentation() {
		return fractionRepresentation;
	}

	/**
	 * Set the fraction internal representation
	 *  
	 * @param representation object containing the representation
	 */
	public void setFractionRepresentation(Representation representation) {
		fractionRepresentation = representation;
	}

	// Instance data
	private int numberBits;
	private boolean negative;
	private Representation decimalRepresentation;
	private Representation fractionRepresentation;
}

/**
 * Class used to convert a binary representation into a CSD representation
 * 
 * @author Craig A. Lindley
 */
class CSD {

	/**
	 * CSD class constructor
	 * 
	 * @param holder Holder object that will be manipulated
	 */
	public CSD(final Holder holder) {

		// Extract info from holder
		numberBits = holder.getNumberBits();
		
		// Get both representations
		decimalRepresentation = holder.getDecimalRepresentation();
		fractionRepresentation = holder.getFractionRepresentation();

		// Create a single buffer to combine both representations
		fullRep = new int [numberBits * 2];
		
		// Combine both representations into a single one
		for (int i = 0; i < numberBits; i++) {
			fullRep[i] = fractionRepresentation.getBit(i);
			fullRep[i + numberBits] = decimalRepresentation.getBit(i);
		}
	}
	
	/**
	 * Scan representation for the next one bit
	 * 
	 * @return bit number if a one is found or -1 if not.
	 */
	private int scanForOne() {
		
		while (bitIndex < (numberBits * 2)) {
			if (fullRep[bitIndex] == 1) {
				return bitIndex++;
			}	
			bitIndex++;
		}
		return -1;
	}

	/**
	 * Peek to see if there is another one in the representation
	 * 
	 * @return true if there is another one bit; false otherwise
	 */
	private boolean peekForOne() {
		
		// Save the current index
		int savedIndex = bitIndex;
		
		// Scan for a one
		int bn = scanForOne();

		// Put the index back
		bitIndex = savedIndex;
		
		return (bn != -1); 
	}
	
	/**
	 * Create an array of integers destined to replace the run in the
	 * representation.
	 * 
	 * @param count is the number of ones in the run
	 * 
	 * @return an array of ints containing the run replacement.
	 */
	private int [] createReplacementRun(final int count) {
		
		// Replacement is always one longer than the run 
		int [] replacement = new int [count + 1]; 
		
		// LS position always contains the -1 token
		replacement [0] = -1;
		
		// The MS position always contains a one
		replacement [count] = 1;

		// The positions between the LS and MS contain zeros
		for (int i = 0; i < count - 1 ; i++) {
			replacement [i + 1] = 0;
		}
		return replacement;
	}
	
	/**
	 * Scans the representation replacing any runs of ones found
	 * 
	 * @return true if a replacement was made; false if not
	 */
	private boolean replaceRun() {
		
		// Always start at the beginning of the representation
		bitIndex = 0;

		// Count of ones in run
		int count = 0;
		
		int runIndex = -1;

		// Look for a one in the representation
		int bn1 = scanForOne();
		
		while (true) {
			if (bn1 == -1) {
				break;
			}
			if (! peekForOne()) {
				break;
			}
			// Two ones were found. Are they consecutive ?
			int bn2 = scanForOne();
			if ((bn2 - bn1) != 1) {
				// Ones not consecutive so try again
				bn1 = bn2;
				continue;
			}
			// We have a run of at least two ones
			runIndex = bn1;
			count = 2;
			
			// See if the run is longer
			while (true) {
				if (! peekForOne()) {
					break;
				}
				int bn3 = scanForOne();
				if ((bn3 - bn2) != 1) {
					break;
				}
				// Found another one
				count++;
				bn2 = bn3;
			}
			if (count != 0) {
				break;
			}
		}
		// Was a run found ?
		if (count != 0) {
			// Yes a run was found. Create replacement
			int [] replacementRun = createReplacementRun(count);
			
			// Check for array overflow
			if ((runIndex + count) >= (numberBits * 2)) {
				count--;
			}
			// Move the replacement into place
			for (int i = 0; i < count + 1; i++) {
				fullRep[runIndex + i] = replacementRun[i];
			}
			// Indicate a replacement was made
			return true;
		}
		// A run was not found
		return false;
	}
	
	/**
	 * Convert a normal representation into a CSD representation.
	 * A conversion is performed on both the decimal and fraction
	 * portions of the representation (as long as they are not zero).
	 */
	public void convertToCSD() {
		
		// Do replacements until no more are possible
		boolean runReplaced = replaceRun();
		while (runReplaced) {
			runReplaced = replaceRun();
		}
		// Separate the converted fullRep into the two component representations
		for (int i = 0; i < numberBits; i++) {
			fractionRepresentation.setBit(i, fullRep[i]);
			decimalRepresentation.setBit(i, fullRep[i + numberBits]);
		}
	}
	
	// Instance data
	private Representation decimalRepresentation;
	private Representation fractionRepresentation;
	private int numberBits;
	private int [] fullRep;
	private int bitIndex;
}

/**
 * Class which provides methods for extracting information from the 
 * representations contained within a holder.
 * 
 * @author Craig A. Lindley
 */
class Parser {
	
	/**
	 * Class constructor for Parser class
	 * 
	 * @param holder Holder object containing a representation
	 * @param doDecimal true if the decimal portion of the representation
	 * should be processed. false if the fraction portion should be
	 * processed.
	 */
	public Parser(final Holder holder, final boolean doDecimal) {
		
		this.holder = holder;
		this.doDecimal = doDecimal;

		// Extract info for ease of access
		numberBits = holder.getNumberBits();
		
		// Reset bit index
		reset();
	}

	/**
	 * Given a bit number return the value of the bit from the appropriate representation
	 * 
	 * @return bit value at bitNumber from set {0, 1, -1}
	 */
	public int getBit(final int bitNumber) {
		
		Representation rep = (doDecimal) ? 
				holder.getDecimalRepresentation() : holder.getFractionRepresentation();
		
		return rep.getBit(bitNumber);
	}
	
	/**
	 * Scan representation in the appropriate direction to find the next one bit.
	 * This method retains state (bitIndex) between calls. Call reset to start
	 * scan over from the beginning.
	 * 
	 * @return bitIndex if another one is found or -1 if end of scan.
	 */
	public int scanForOne() {
		
		if (doDecimal) {
			Representation rep = holder.getDecimalRepresentation();
			
			while (bitIndex >= 0) {
				if (rep.getBit(bitIndex) != 0) {
					return bitIndex--;
				}
				bitIndex--;
			}
			return -1;
			
		} else {

			Representation rep = holder.getFractionRepresentation();
			
			while (bitIndex < numberBits) {
				if (rep.getBit(bitIndex) != 0) {
					return bitIndex++;
				}
				bitIndex++;
			}
			return -1;
		}
	}
	
	/**
	 * Peek to see if there is another one in the representation
	 * 
	 * @return true if there is another one bit; false otherwise
	 */
	public boolean peekForOne() {
		
		// Save the current index
		int savedIndex = bitIndex;
		
		// Scan for a one
		int bn = scanForOne();

		// Put the index back
		bitIndex = savedIndex;
		
		return (bn != -1); 
	}
	
	/**
	 * Calling reset sets bitIndex to start scanning
	 * over from the beginning.
	 */
	public void reset() {
		if (doDecimal) {
			bitIndex = numberBits - 1;
		} else {
			bitIndex = 0;
		}
	}
	
	// Instance data
	private Holder holder;
	private boolean doDecimal;
	private int numberBits;
	private int bitIndex;
}

/**
 * Class used to generate the Horner equations for multiplication expressed in
 * terms of shifts and addes.
 * 
 * @author Craig A. Lindley
 */
public class Horner {
	
	private static final String CRLF 			= "\r\n";
	private static final int GEN_EQU			= 1;
	private static final int GEN_CODE 			= 2;
	private static final int GEN_EQU_AND_CODE	= 3;
	
		
	/**
	 * Class constructor
	 * 
	 * @param useCSD should be set true if CSD is to be used as optimization
	 * false if not.
	 */
	public Horner(final int numberBits, final boolean useCSD) {
		
		this.numberBits = numberBits;
		this.useCSD = useCSD;
	}
	
	/**
	 * Determine last term of fractional representation
	 * 
	 * @param guardBit the guard bit associated with fraction
	 * @param signBit the sign bit associated with the fraction
	 * 
	 * @return String containing last term addin if any
	 */
	private String determineLastTermOfFraction(
			final boolean guardBit, final boolean signBit) {
		
		String ret = "";
		
		if (! guardBit) {
			// guard bit false
			if (signBit) {
				ret = " - X";
			}
			
		}	else	{
			// guard bit true
			if (! signBit) {
				ret = " + X";
			}
		}
		return ret;
	}
	
	/**
	 * Generate the Horner equations for decimal portion of the multiplier
	 * contained in the holder.
	 *  
	 * @param sb StringBuffer object to generate equations in
	 * @param holder Holder object containing a representation
	 */
	private void generateDecimalEquations(
				final StringBuffer sb, final Holder holder) {
		
		// If we get here we know there are ones in representation

		// Extract info from representation
		Representation representation = holder.getDecimalRepresentation();
		boolean signBit = representation.getSignBit();
		String sign = (signBit) ? "-" : "";
		
		// Create a parser for the decimal portion of the representation
		Parser p = new Parser(holder, true);
		
		// Get the next one
		int bn1 = p.scanForOne();

		// temp variable index
		int tempIndex = -1;

		while (true) {
			// Is there another one in the representation ?
			if (p.peekForOne()) {
				// Yes there is another one so get it
				int bn2 = p.scanForOne();
				int distance = bn1 - bn2;
				tempIndex = (tempIndex == -1) ? 1 : tempIndex + 1;
				if (tempIndex == 1) {
					// The first time
					sb.append("T" + tempIndex + " = " + sign + "X * 2^" + distance);
				} else {
					sb.append("T" + tempIndex + " = T" + (tempIndex - 1) + " * 2^" + distance);
				}
				int bit = p.getBit(bn2);
				if (bit == 1) {
					sb.append(" + X");
				} else {
					sb.append(" - X");
				}
				sb.append(CRLF);

				bn1 = bn2;

			} else {

				// No other ones
				int bitWeight = bn1;

				if (tempIndex == -1) {
					sb.append("Decimal result = " + sign + "X * 2^" + bitWeight);
				} else {
					sb.append("Decimal result = T" + tempIndex);
					sb.append(" * 2^" + bitWeight);
				}
				sb.append(CRLF);
				break;
			}
		}
	}

	/**
	 * Generate the Horner equations for fraction portion of the multiplier
	 * contained in the holder.
	 *  
	 * @param holder Holder object containing a representation
	 * @param sb StringBuffer object to generate equations in
	 * @param wasDecimalPart true if number has a decimal part; false
	 * if number was only fractional part.
	 */
	private void generateFractionEquations(
			final StringBuffer sb, final Holder holder, final boolean wasDecimalPart) {

		// If we get here we know there are ones in representation

		// Extract info from representation
		Representation representation = holder.getFractionRepresentation();
		boolean guardBit = representation.getGuardBit();
		boolean signBit = representation.getSignBit();

		String finalTerm = (wasDecimalPart) ? "" : determineLastTermOfFraction(guardBit, signBit);
		
		// Create a parser for the fraction portion of the representation
		Parser p = new Parser(holder, false);
		
		// Get the next one
		int bn1 = p.scanForOne();

		// temp variable index
		int tempIndex = -1;

		while (true) {
			// Is there another one in the representation ?
			if (p.peekForOne()) {
				// Yes there is another one so get it
				int bn2 = p.scanForOne();
				int distance = bn1 - bn2;
				tempIndex = (tempIndex == -1) ? 1 : tempIndex + 1;
				if (tempIndex == 1) {
					// The first time
					sb.append("T" + tempIndex + " = X * 2^" + distance);
				} else {
					sb.append("T" + tempIndex + " = T" + (tempIndex - 1) + " * 2^" + distance);
				}
				int bit = p.getBit(bn2);
				if (bit == 1) {
					sb.append(" + X");
				} else {
					sb.append(" - X");
				}
				sb.append(CRLF);

				bn1 = bn2;

			} else {

				// No other ones
				int bitWeight = bn1 - numberBits;

				if (tempIndex == -1) {
					sb.append("Fraction result = X * 2^" + bitWeight);
				} else {
					sb.append("Fraction result = T" + tempIndex);
					sb.append(" * 2^" + bitWeight);
				}
				sb.append(finalTerm);
				
				sb.append(CRLF);
				break;
			}
		}
	}
		
	/**
	 * Generate the MSP430 code for the decimal portion of the multiplier
	 * contained in the holder.
	 *  
	 * @param sb StringBuffer object to generate code in
	 * @param holder Holder object containing a representation
	 */
	private void generateDecimalCode(final StringBuffer sb, final Holder holder) {
		
		// If we get here we know there are ones in representation

		// Extract info from representation
		Representation representation = holder.getDecimalRepresentation();

		// Is the number negative ?
		if (representation.getSignBit()) {
			// Do 2's comp of acc for -X term
			sb.append("\tinv\tacc" + CRLF);
			sb.append("\tinc\tacc" + CRLF);
		}
		// Create a parser for the decimal portion of the representation
		Parser p = new Parser(holder, true);
		
		// Get the next one
		int bn1 = p.scanForOne();

		while (true) {
			// Is there another one in the representation ?
			if (p.peekForOne()) {
				// Yes there is another one so get it
				int bn2 = p.scanForOne();
				int distance = bn1 - bn2;
				for (int i = 0; i < distance; i++) {
					sb.append("\trla\tacc" + CRLF);
				}
				int bit = p.getBit(bn2);
				if (bit == 1) {
					sb.append("\tadd\tin,acc");
				} else {
					sb.append("\tsub\tin,acc");
				}
				sb.append(CRLF);

				bn1 = bn2;

			} else {

				// No other ones
				int bitWeight = bn1;
				
				for (int i = 0; i < bitWeight; i++) {
					sb.append("\trla\tacc" + CRLF);
				}
				sb.append("\tmov\tacc,out" + CRLF);
				break;
			}
		}
	}
	
	/**
	 * Generate the MSP430 code for fraction portion of the multiplier
	 * contained in the holder.
	 *  
	 * @param holder Holder object containing a representation
	 * @param sb StringBuffer object to generate equations in
	 * @param wasDecimalPart true if number has a decimal part; false
	 * if number was only fractional part.
	 */
	private void generateFractionCode(
			final StringBuffer sb, final Holder holder, final boolean wasDecimalPart) {

		// If we get here we know there are ones in representation

		// Extract info from representation
		Representation representation = holder.getFractionRepresentation();
		boolean guardBit = representation.getGuardBit();
		boolean signBit = representation.getSignBit();

		String finalTerm = (wasDecimalPart) ? "" : determineLastTermOfFraction(guardBit, signBit);
		
		// Create a parser for the fraction portion of the representation
		Parser p = new Parser(holder, false);
		
		// Get the next one
		int bn1 = p.scanForOne();

		while (true) {
			// Is there another one in the representation ?
			if (p.peekForOne()) {
				// Yes there is another one so get it
				int bn2 = p.scanForOne();
				int distance = Math.abs(bn1 - bn2);
				for (int i = 0; i < distance; i++) {
					sb.append("\trra\tacc" + CRLF);
				}
				int bit = p.getBit(bn2);
				if (bit == 1) {
					sb.append("\tadd\tin,acc");
				} else {
					sb.append("\tsub\tin,acc");
				}
				sb.append(CRLF);

				bn1 = bn2;

			} else {

				// No other ones
				int bitWeight = Math.abs(bn1 - numberBits);
				for (int i = 0; i < bitWeight; i++) {
					sb.append("\trra\tacc" + CRLF);
				}
				int plusIndex = finalTerm.indexOf("+");
				int minusIndex = finalTerm.indexOf("-");
				
				if (plusIndex != -1) {
					sb.append("\tadd\tin,acc");
				}
				if (minusIndex != -1) {
					sb.append("\tsub\tin,acc");
				}
				sb.append(CRLF);
				break;
			}
		}
	}
		
	/**
	 * Generate the Horner equations for the multiplier contained
	 * in the holder.
	 * 
	 * @param holder Holder object containing the decimal and fraction
	 * representation of the multiplier
	 * 
	 * @return A String containing the Horner equations
	 */
	private String generateEquations(final Holder holder) {
		
		// StringBuffer to hold generated equations
		StringBuffer sb = new StringBuffer();
		
		boolean wasDecimalPart = false;
		boolean wasFractionPart = false;
		
		// Are there bits in decimal representation ?
		if (! holder.getDecimalRepresentation().zero()) {
			// Yes so generate equations for the decimal representation
			generateDecimalEquations(sb, holder);
			wasDecimalPart = true;
			sb.append(CRLF);
		}

		// Are there bits in fraction representation ?
		if (! holder.getFractionRepresentation().zero()) {
			// Yes so generate equations for the fraction representation
			generateFractionEquations(sb, holder, wasDecimalPart);
			wasFractionPart = true;
			sb.append(CRLF);
		}

		if (wasDecimalPart && wasFractionPart) {
			sb.append("Total Result = Decimal result + Fraction result" + CRLF);
		}
		
		return sb.toString();
	}
	
	/**
	 * Generate the MSP430 code for the multiplier contained
	 * in the holder.
	 * 
	 * @param holder Holder object containing the decimal and fraction
	 * representation of the multiplier
	 * 
	 * @return A String containing the generated code
	 */
	private String generateCode(final Holder holder) {
		
		// StringBuffer to hold generated equations
		StringBuffer sb = new StringBuffer();
		sb.append("Code conventions:" + CRLF);
		sb.append("\tRegister \"in\" has the X value to be multiplied" + CRLF);
		sb.append("\tRegister \"acc\" is the accumulator used for temporary values" + CRLF);
		sb.append("\tRegister \"out\" holds the final result" + CRLF + CRLF);
		
		sb.append("\r\tmov\tin,acc" + CRLF);
		
		boolean wasDecimalPart = false;
		boolean wasFractionPart = false;
		
		// Are there bits in decimal representation ?
		if (! holder.getDecimalRepresentation().zero()) {
			// Yes so generate code for the decimal representation
			generateDecimalCode(sb, holder);
			wasDecimalPart = true;
			sb.append(CRLF);
		}

		// Are there bits in fraction representation ?
		if (! holder.getFractionRepresentation().zero()) {
			// Yes there are bits in fraction representation
			
			// Reload acc with in value if appropriate
			if (wasDecimalPart) { 
				sb.append("\r\tmov\tin,acc" + CRLF);
			}
			// Generate code for the fraction representation
			generateFractionCode(sb, holder, wasDecimalPart);
			wasFractionPart = true;
		}

		if (wasDecimalPart && wasFractionPart) {
			sb.append("\tadd\tacc,out" + CRLF);
		}	else	{
			sb.append("\tmov\tacc,out" + CRLF);
		}
		
		return sb.toString();
	}
	

	/**
	 * Generate and display the output for a given number
	 * 
	 * @param number multiplier to generate the output for
	 * @param out 0 is no output, 1 is code output, 2 is equation output
	 * and 3 is both code and equation output. Code output is for the
	 * MSP430 microcontroller.
	 */
	public void displayOutput(final double number, final int out, final boolean bin) {
		
		// Do some setup
		// Create a holder for the number. This converts number into 
		// binary representaion 
		Holder h = new Holder(numberBits, number);

		System.out.println(CRLF + "Multiplier: " + number + " Bits of precision: " + numberBits + CRLF);

		// Did the user want to see the binary ?
		if (bin) {
			System.out.println("Binary representation: " + h + CRLF);
		}

		// Use CSD as an optimization ?
		if (useCSD) {
			// Do a CSD conversion of the representation
			CSD c = new CSD(h);
			c.convertToCSD();
			System.out.println("CSD representation: " + h + CRLF);
		}
		// Should Horner equations be generated ?
		if (out == GEN_EQU) {
			// Generate and display the Horner equations for multiplication by the number
			System.out.println("Horner equations: " + CRLF + generateEquations(h));

		}	else if (out == GEN_CODE) {
			// Generate and display the MSP430 code for multiplication by the number
			System.out.println("MSP430 code: " + CRLF + generateCode(h));

		}	else if (out == GEN_EQU_AND_CODE) {
			// Generate and display the Horner equations for multiplication by the number
			System.out.println("Horner equations: " + CRLF + generateEquations(h));

			// Generate and display the MSP430 code for multiplication by the number
			System.out.println("MSP430 code: " + CRLF + generateCode(h));
		}
	}
	
	/**
	 * Print out a usage string and terminate program operation
	 */
	public static void usage(final String msg) {

		StringBuffer sb = new StringBuffer();
		
		sb.append(CRLF);
		if (msg != null) {
			sb.append(msg + CRLF + CRLF);
		}
		sb.append("Horner.java V1.0 copyright 2007 by: Craig A. Lindley" + CRLF);
		sb.append("Generates Horner equations and MSP430 code for multiplication using only shifts and adds" + CRLF + CRLF);
		sb.append("Program usage is as follows:" + CRLF);
		sb.append("  java Horner [nocsd] [csd] [bin] [16bit] [12bit] [10bit] [code] [equ] [both] [help | ?] multiplier" + CRLF);
		sb.append("  Where:" + CRLF);
		sb.append("    nocsd - use binary not csd numbers in the calculations" + CRLF);
		sb.append("    csd - use csd optimizations in the calculations" + CRLF);
		sb.append("    bin - show binary representation of multiplier" + CRLF);
		sb.append("    16bit - use 16 bit binary precision" + CRLF);
		sb.append("    12bit - use 12 bit binary precision" + CRLF);
		sb.append("    10bit - use 10 bit binary precision" + CRLF);
		sb.append("    code - generate the MSP430 code for the multiplication" + CRLF);
		sb.append("    equ - generate Horner equations for the multiplication" + CRLF);
		sb.append("    both - generate the MSP430 code and the equations for the multiplication" + CRLF);
		sb.append("    help or ? - displays this message" + CRLF);
		sb.append("    multiplier - the floating point number being multiplied. The multiplier must" + CRLF);
		sb.append("                 be the final argument." + CRLF + CRLF);
		sb.append("  Program defaults: 16bit csd both" + CRLF);

		if (msg != null) {
			sb.append("Please try again" + CRLF);
		}
		sb.append(CRLF);
		
		System.err.println(sb.toString());
		System.exit(-1);
	}
	
	/**
	 * Main entry point used for testing
	 * 
	 * @param args double number to generate outputx  for. If no
	 * args are specified canned numbers are used for equation generation.
	 * Numbers and equations are written to stdout.
	 */
	public static void main(final String [] args) {

		// Default values are 16 bits, csd mode, equation and code output
		boolean mode = true;		// mode == true is csd
		int numberBits = 16;		// number of bits in representations
		int out = 3;				// out == 3 means generate both equ and code
		boolean bin = false;		// don't show the binary

		// Get the count of user args
		int numberArgs = args.length;
		
		if (numberArgs == 0) {
			usage("No command line arguments specified");
		}
		
		// Process all user args
		for (int i = 0; i < numberArgs - 1; i++) {
			// Get the command line argument
			String arg = args[i].toLowerCase();
			
			if (arg.indexOf("nocsd") != -1) {
				mode = false;
			}	else if (arg.indexOf("csd") != -1) {
				mode = true;
			}	else if (arg.indexOf("bin") != -1) {
				bin = true;
			}	else if (arg.indexOf("16bit") != -1) {
				numberBits = 16;
			}	else if (arg.indexOf("12bit") != -1) {
				numberBits = 12;
			}	else if (arg.indexOf("10bit") != -1) {
				numberBits = 10;
			}	else if (arg.indexOf("equ") != -1) {
				out = GEN_EQU;
			}	else if (arg.indexOf("code") != -1) {
				out = GEN_CODE;
			}	else if (arg.indexOf("both") != -1) {
				out = GEN_EQU_AND_CODE;
			}	else if (arg.indexOf("help") != -1) {
				usage(null);
			}	else if (arg.indexOf("?") != -1) {
				usage(null);
			}	else {
				usage("Unknown argument \"" + arg + "\" specified");
			}
		}
		// Get the final user specified command line argument
		String finalArg = args[numberArgs - 1];

		// Final argument must be request for help or double number multiplier
		if ((finalArg.indexOf("help") != -1) || (finalArg.indexOf("?") != -1)) {
			usage(null);
		}
		
		// Verify that the final arg is a valid double number 
		double multiplier = 0;
		try {
			multiplier = new Double(finalArg).doubleValue();
		} catch (NumberFormatException nfe) {
			usage("Error: final multiplier argument \"" + finalArg + "\" not a double number");
		}
		// Instantiate Horner object and go
		Horner h = new Horner(numberBits, mode);
		h.displayOutput(multiplier, out, bin);
	}
	
	// Instance data
	private int numberBits;
	private boolean useCSD;
}